@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2021 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2022 sqlmap developers (https://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -167,8 +167,27 @@ class WichmannHill(random.Random):
 
 def patchHeaders(headers):
     if headers is not None and not hasattr(headers, "headers"):
+        if isinstance(headers, dict):
+            class _(dict):
+                def __getitem__(self, key):
+                    for key_ in self:
+                        if key_.lower() == key.lower():
+                            return super(_, self).__getitem__(key_)
+
+                    raise KeyError(key)
+
+                def get(self, key, default=None):
+                    try:
+                        return self[key]
+                    except KeyError:
+                        return default
+
+            headers = _(headers)
+
         headers.headers = ["%s: %s\r\n" % (header, headers[header]) for header in headers]
 
+    return headers
+
 def cmp(a, b):
     """
     >>> cmp("a", "b")
@@ -257,3 +276,9 @@ if sys.version_info >= (3, 0):
 else:
     xrange = xrange
     buffer = buffer
+
+try:
+    from packaging import version
+    LooseVersion = version.parse
+except ImportError:
+    from distutils.version import LooseVersion
