@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2020 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2022 sqlmap developers (https://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -46,6 +46,7 @@ optDict = {
         "proxy": "string",
         "proxyCred": "string",
         "proxyFile": "string",
+        "proxyFreq": "integer",
         "tor": "boolean",
         "torPort": "integer",
         "torType": "string",
@@ -53,6 +54,7 @@ optDict = {
         "delay": "float",
         "timeout": "float",
         "retries": "integer",
+        "retryOn": "string",
         "rParam": "string",
         "safeUrl": "string",
         "safePost": "string",
@@ -62,6 +64,7 @@ optDict = {
         "csrfToken": "string",
         "csrfUrl": "string",
         "csrfMethod": "string",
+        "csrfData": "string",
         "csrfRetries": "integer",
         "forceSSL": "boolean",
         "chunked": "boolean",
@@ -241,6 +244,7 @@ optDict = {
         "dependencies": "boolean",
         "disableColoring": "boolean",
         "listTampers": "boolean",
+        "noLogging": "boolean",
         "offline": "boolean",
         "purge": "boolean",
         "resultsFile": "string",
